/*
Copyright (C) 2001, 2006 United States Government
as represented by the Administrator of the
National Aeronautics and Space Administration.
All Rights Reserved.
*/
package gov.nasa.worldwind.globes;

import gov.nasa.worldwind.*;
import gov.nasa.worldwind.avlist.AVKey;

/**
 * @author Tom Gaskins
 * @version $Id: Earth.java 9411 2009-03-16 21:03:25Z tgaskins $
 */

public class Earth extends EllipsoidalGlobe
{
    public static final double WGS84_EQUATORIAL_RADIUS = 6378137.0; // ellipsoid equatorial getRadius, in meters
    public static final double WGS84_POLAR_RADIUS = 6356752.3; // ellipsoid polar getRadius, in meters
    public static final double WGS84_ES = 0.00669437999013; // eccentricity squared, semi-major axis

    public Earth()
    {
        super(WGS84_EQUATORIAL_RADIUS, WGS84_POLAR_RADIUS, WGS84_ES, (ElevationModel)
            WorldWind.createComponentFromConfigurationFactory(AVKey.ELEVATION_MODEL_FACTORY,
                new String[]
                    {
                        AVKey.EARTH_ELEVATION_MODEL_CONFIG_FILE, // first try to create from a config file
                        AVKey.EARTH_ELEVATION_MODEL_CLASS_NAME // otherwise try to create from a class name
                    }));
    }

    public String toString()
    {
        return "Earth";
    }
}
