/*
Copyright (C) 2001, 2008 United States Government
as represented by the Administrator of the
National Aeronautics and Space Administration.
All Rights Reserved.
*/

package gov.nasa.worldwind.layers;

import gov.nasa.worldwind.render.DrawContext;
import gov.nasa.worldwind.render.markers.Marker;
import gov.nasa.worldwind.render.markers.MarkerRenderer;
import gov.nasa.worldwind.terrain.SectorGeometryList;
import gov.nasa.worldwind.util.Logging;

/**
 * @author tag
 * @version $Id: MarkerLayer.java 6835 2008-09-26 03:08:27Z tgaskins $
 */
public class MarkerLayer extends AbstractLayer
{
    private MarkerRenderer markerRenderer = new MarkerRenderer();
    private Iterable<Marker> markers;

    public MarkerLayer()
    {
    }

    public MarkerLayer(Iterable<Marker> markers)
    {
        this.markers = markers;
    }

    public Iterable<Marker> getMarkers()
    {
        return markers;
    }

    public void setMarkers(Iterable<Marker> markers)
    {
        this.markers = markers;
    }

    public double getElevation()
    {
        return this.getMarkerRenderer().getElevation();
    }

    public void setElevation(double elevation)
    {
        this.getMarkerRenderer().setElevation(elevation);
    }

    public boolean isOverrideMarkerElevation()
    {
        return this.getMarkerRenderer().isOverrideMarkerElevation();
    }

    public void setOverrideMarkerElevation(boolean overrideMarkerElevation)
    {
        this.getMarkerRenderer().setOverrideMarkerElevation(overrideMarkerElevation);
    }

    public boolean isKeepSeparated()
    {
        return this.getMarkerRenderer().isKeepSeparated();
    }

    public void setKeepSeparated(boolean keepSeparated)
    {
        this.getMarkerRenderer().setKeepSeparated(keepSeparated);
    }

    public boolean isEnablePickSizeReturn()
    {
        return this.getMarkerRenderer().isEnablePickSizeReturn();
    }

    public void setEnablePickSizeReturn(boolean enablePickSizeReturn)
    {
        this.getMarkerRenderer().setEnablePickSizeReturn(enablePickSizeReturn);
    }

    /**
     * Opacity is not applied to layers of this type because each marker has an attribute set with opacity control.
     *
     * @param opacity the current opacity value, which is ignored by this layer.
     */
    @Override
    public void setOpacity(double opacity)
    {
        super.setOpacity(opacity);
    }

    /**
     * Returns the layer's opacity value, which is ignored by this layer because each of its markers has an attribute
     * with its own opacity control.
     *
     * @return The layer opacity, a value between 0 and 1.
     */
    @Override
    public double getOpacity()
    {
        return super.getOpacity();
    }

    protected MarkerRenderer getMarkerRenderer()
    {
        return markerRenderer;
    }

    protected void setMarkerRenderer(MarkerRenderer markerRenderer)
    {
        this.markerRenderer = markerRenderer;
    }

    protected void doRender(DrawContext dc)
    {
        this.draw(dc, null);
    }

    @Override
    protected void doPick(DrawContext dc, java.awt.Point pickPoint)
    {
        this.draw(dc, pickPoint);
    }

    protected void draw(DrawContext dc, java.awt.Point pickPoint)
    {
        if (this.markers == null)
            return;

        if (dc.getVisibleSector() == null)
            return;

        SectorGeometryList geos = dc.getSurfaceGeometry();
        if (geos == null)
            return;
//
//        if (this.visList == null)
//            this.visList = this.sortMarkersByRegion(dc, this.markers);
//
//        VisListIterable iter = new VisListIterable(dc, this.visList);

        if (dc.isPickingMode())
            this.getMarkerRenderer().pick(dc, this.markers, pickPoint, this);
        else
            this.getMarkerRenderer().render(dc, this.markers);
    }
//
//    private Map<SectorInfo, List<Marker>> visList;
//
//    private static class SectorInfo
//    {
//        private Sector sector;
//        private Extent extent;
//    }
//
//    private static class VisListIterable implements Iterable<Marker>
//    {
//        private DrawContext dc;
//        private Map<SectorInfo, List<Marker>> visList = new HashMap<SectorInfo, List<Marker>>();
//
//        private VisListIterable(DrawContext dc, Map<SectorInfo, List<Marker>> visList)
//        {
//            this.visList = visList;
//            this.dc = dc;
//        }
//
//        public Iterator<Marker> iterator()
//        {
//            return new VisListMarkerIterator(dc, visList);
//        }
//    }
//
//    private static class VisListMarkerIterator implements Iterator<Marker>
//    {
//        private DrawContext dc;
//        private Iterator<Map.Entry<SectorInfo, List<Marker>>> curRegions;
//        private Iterator<Marker> curMarkers;
//        private Marker nextMarker;
//
//        public VisListMarkerIterator(DrawContext dc, Map<SectorInfo, List<Marker>> visList)
//        {
//            this.curRegions = visList.entrySet().iterator();
//            this.dc = dc;
//        }
//
//        public boolean hasNext()
//        {
//            return loadNext();
//        }
//
//        public boolean loadNext()
//        {
//            if (curMarkers != null && curMarkers.hasNext())
//            {
//                nextMarker = curMarkers.next();
//            }
//            else if (curRegions != null)
//            {
//                while (curRegions.hasNext())
//                {
//                    Map.Entry<SectorInfo, List<Marker>> region = curRegions.next();
//
//                    List<Marker> markerList = region.getValue();
//                    if (markerList == null)
//                        continue;
//
//                    if (!isVisible(region.getKey()))
//                        continue;
//
//                    curMarkers = markerList.iterator();
//                    if (curMarkers.hasNext())
//                    {
//                        nextMarker = curMarkers.next();
//                        break;
//                    }
//                }
//            }
//
//            return nextMarker != null;
//        }
//
//        public Marker next()
//        {
//            if (nextMarker == null)
//                throw new NoSuchElementException();
//
//            Marker m = nextMarker;
//            nextMarker = null;
//            return m;
//        }
//
//        private boolean isVisible(SectorInfo s)
//        {
//            return s.extent.intersects(dc.getView().getFrustumInModelCoordinates());
//        }
//
//        public void remove()
//        {
//            throw new UnsupportedOperationException();
//        }
//    }
//
//    protected Map<SectorInfo, List<Marker>> sortMarkersByRegion(DrawContext dc, Iterable<Marker> markers)
//    {
//        List<SectorInfo> sectors = new ArrayList<SectorInfo>();
//        Map<SectorInfo, List<Marker>> visList = new HashMap<SectorInfo, List<Marker>>();
//
//        int delta = 5;
//        for (int lat = -90; lat < 90; lat += delta)
//        {
//            for (int lon = -180; lon < 180; lon += delta)
//            {
//                SectorInfo si = new SectorInfo();
//                si.sector = Sector.fromDegrees(lat, lat + delta, lon, lon + delta);
//                si.extent = dc.getGlobe().computeBoundingCylinder(1d, si.sector);
//                sectors.add(si);
//            }
//        }
//
//        for (Marker marker : markers)
//        {
//            for (SectorInfo region : sectors)
//            {
//                if (region.sector.contains(marker.getPosition()))
//                {
//                    List<Marker> regionMarkers = visList.get(region);
//                    if (regionMarkers == null)
//                    {
//                        regionMarkers = new ArrayList<Marker>();
//                        visList.put(region, regionMarkers);
//                    }
//                    regionMarkers.add(marker);
//                    break;
//                }
//            }
//        }
//
//        return visList;
//    }

    @Override
    public String toString()
    {
        return Logging.getMessage("layers.MarkerLayer.Name");
    }
}
