/* Copyright (C) 2001, 2009 United States Government as represented by
the Administrator of the National Aeronautics and Space Administration.
All Rights Reserved.
*/
package gov.nasa.worldwind.util;

import javax.media.opengl.GL;

/**
 * @author dcollins
 * @version $Id: OGLOrtho2DSupport.java 10634 2009-04-29 03:52:29Z dcollins $
 */
public class OGLOrtho2DSupport
{
    protected java.awt.Rectangle viewport;
    protected OGLStackHandler stackHandler;

    public OGLOrtho2DSupport()
    {
        this.stackHandler = new OGLStackHandler();
    }

    public java.awt.Rectangle getViewport()
    {
        return (this.viewport != null) ? new java.awt.Rectangle(this.viewport) : null;
    }

    public void beginRendering(GL gl, int x, int y, int width, int height)
    {
        if (gl == null)
        {
            String message = Logging.getMessage("nullValue.GLIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        this.viewport = new java.awt.Rectangle(x, y, width, height);

        this.stackHandler.pushAttrib(gl,
            GL.GL_CURRENT_BIT
            | GL.GL_ENABLE_BIT
            | GL.GL_VIEWPORT_BIT
            | GL.GL_POLYGON_BIT);

        this.stackHandler.pushTextureIdentity(gl);
        this.stackHandler.pushProjectionIdentity(gl);
        gl.glOrtho(x, x + width, y, y + height, -1, 1);
        this.stackHandler.pushModelviewIdentity(gl);

        gl.glDisable(GL.GL_DEPTH_TEST);
        gl.glEnable(GL.GL_CULL_FACE);
        gl.glCullFace(GL.GL_BACK);
        gl.glViewport(x, y, width, height);
    }

    public void endRendering(GL gl)
    {
        if (gl == null)
        {
            String message = Logging.getMessage("nullValue.GLIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        this.stackHandler.pop(gl);

        this.viewport = null;
    }

    public void clear(GL gl, java.awt.Color color)
    {
        if (gl == null)
        {
            String message = Logging.getMessage("nullValue.GLIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        if (color == null)
        {
            String message = Logging.getMessage("nullValue.ColorIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        float[] compArray = new float[4];
        color.getRGBComponents(compArray);
        // Premultiply color components by the alpha component.
        compArray[0] *= compArray[3];
        compArray[1] *= compArray[3];
        compArray[2] *= compArray[3];

        gl.glClearColor(compArray[0], compArray[1], compArray[2], compArray[3]);
        gl.glClear(GL.GL_COLOR_BUFFER_BIT);
    }
}
